@@ -21,69 +21,9 @@
 struct bits_t bits;
 unsigned long long vramsize;
 int drm_fd = -1;
+char drm_name[10] = ""; // should be radeon or amdgpu
 
-static int drmfilter(const struct dirent *ent) {
-
-	if (ent->d_name[0] == '.')
-		return 0;
-	if (strncmp(ent->d_name, "card", 4))
-		return 0;
-
-	return 1;
-}
-
-static void finddrm(const unsigned char bus) {
-
-	int fd, i;
-	struct dirent **namelist;
-	const int entries = scandir("/dev/dri", &namelist, drmfilter, alphasort);
-	char tmp[160];
-
-	if (entries < 0) {
-		perror("scandir");
-		return;
-	}
-
-	for (i = 0; i < entries; i++) {
-		snprintf(tmp, 160, "/dev/dri/%s", namelist[i]->d_name);
-
-		fd = open(tmp, O_RDWR);
-		if (fd < 0) continue;
-
-		const char *busid = drmGetBusid(fd);
-
-		if (strncmp(busid, "pci:", 4))
-			goto fail;
-
-		busid = strchr(busid, ':');
-		if (!busid) goto fail;
-		busid++;
-
-		busid = strchr(busid, ':');
-		if (!busid) goto fail;
-		busid++;
-
-		unsigned parsed;
-		if (sscanf(busid, "%x", &parsed) != 1)
-			goto fail;
-
-		if (parsed == bus) {
-			drm_fd = fd;
-			break;
-		}
-
-		fail:
-		close(fd);
-	}
-
-	for (i = 0; i < entries; i++) {
-		free(namelist[i]);
-	}
-
-	free(namelist);
-}
-
-unsigned int init_pci(unsigned char bus) {
+unsigned int init_pci(unsigned char bus, const unsigned char forcemem) {
 
 	int ret = pci_system_init();
 	if (ret)
@@ -101,12 +41,15 @@ unsigned int init_pci(unsigned char bus) {
 
 	struct pci_device_iterator *iter = pci_id_match_iterator_create(&match);
 	struct pci_device *dev = NULL;
+	char busid[32];
 
 	while ((dev = pci_device_next(iter))) {
 		pci_device_probe(dev);
 		if ((dev->device_class & 0x00ffff00) != 0x00030000 &&
 			(dev->device_class & 0x00ffff00) != 0x00038000)
 			continue;
+		snprintf(busid, sizeof(busid), "pci:%04x:%02x:%02x.%u",
+				dev->domain, dev->bus, dev->dev, dev->func);
 		if (!bus || bus == dev->bus)
 			break;
 	}
@@ -116,8 +59,9 @@ unsigned int init_pci(unsigned char bus) {
 	if (!dev)
 		die(_("Can't find Radeon cards"));
 
+	const unsigned int device_id = dev->device_id;
 	int reg = 2;
-	if (getfamily(dev->device_id) >= BONAIRE)
+	if (getfamily(device_id) >= BONAIRE)
 		reg = 5;
 
 	if (!dev->regions[reg].size) die(_("Can't get the register area size"));
@@ -125,19 +69,31 @@ unsigned int init_pci(unsigned char bus) {
 //	printf("Found area %p, size %lu\n", area, dev->regions[reg].size);
 
 	// DRM support for VRAM
-	if (bus)
-		finddrm(bus);
-	else if (access("/dev/dri/card0", F_OK) == 0)
-		drm_fd = open("/dev/dri/card0", O_RDWR);
-	else if (access("/dev/ati/card0", F_OK) == 0) // fglrx path
+	drm_fd = drmOpen(NULL, busid);
+	if (drm_fd >= 0) {
+		drmVersionPtr ver = drmGetVersion(drm_fd);
+		if (strcmp(ver->name, "radeon") != 0 && strcmp(ver->name, "amdgpu") != 0) {
+			close(drm_fd);
+			drm_fd = -1;
+		}
+		strcpy(drm_name, ver->name);
+		drmFreeVersion(ver);
+	}
+	if (drm_fd < 0 && access("/dev/ati/card0", F_OK) == 0) // fglrx path
 		drm_fd = open("/dev/ati/card0", O_RDWR);
 
 	use_ioctl = 0;
 	if (drm_fd >= 0) {
+		authenticate_drm(drm_fd);
 		uint32_t rreg = 0x8010;
 		use_ioctl = get_drm_value(drm_fd, RADEON_INFO_READ_REG, &rreg);
 	}
 
+	if (forcemem) {
+		printf(_("Forcing the /dev/mem path.\n"));
+		use_ioctl = 0;
+	}
+
 	if (!use_ioctl) {
 		int mem = open("/dev/mem", O_RDONLY);
 		if (mem < 0) die(_("Cannot access GPU registers, are you root?"));
@@ -152,7 +108,7 @@ unsigned int init_pci(unsigned char bus) {
 		printf(_("Failed to open DRM node, no VRAM support.\n"));
 	} else {
 		drmDropMaster(drm_fd);
-		const drmVersion * const ver = drmGetVersion(drm_fd);
+		drmVersionPtr ver = drmGetVersion(drm_fd);
 
 /*		printf("Version %u.%u.%u, name %s\n",
 			ver->version_major,
@@ -160,27 +116,53 @@ unsigned int init_pci(unsigned char bus) {
 			ver->version_patchlevel,
 			ver->name);*/
 
-		if (ver->version_major != 2 ||
-			ver->version_minor < 36) {
+		if (ver->version_major < 2 ||
+			(ver->version_major == 2 && ver->version_minor < 36)) {
 			printf(_("Kernel too old for VRAM reporting.\n"));
+			drmFreeVersion(ver);
 			goto out;
 		}
+		drmFreeVersion(ver);
 
 		// No version indicator, so we need to test once
+		// We use different codepaths for radeon and amdgpu
+		// We store vram_size and check below if the ret value is sane
+		if (strcmp(drm_name, "radeon") == 0) {
+			struct drm_radeon_gem_info gem;
 
-		struct drm_radeon_gem_info gem;
+			ret = drmCommandWriteRead(drm_fd, DRM_RADEON_GEM_INFO,
+							&gem, sizeof(gem));
+			vramsize = gem.vram_size;
+		} else if (strcmp(drm_name, "amdgpu") == 0) {
+#ifdef ENABLE_AMDGPU
+			struct drm_amdgpu_info_vram_gtt vram_gtt = {};
 
-		ret = drmCommandWriteRead(drm_fd, DRM_RADEON_GEM_INFO,
-						&gem, sizeof(gem));
+			struct drm_amdgpu_info request;
+			memset(&request, 0, sizeof(request));
+			request.return_pointer = (unsigned long) &vram_gtt;
+			request.return_size = sizeof(vram_gtt);
+			request.query = AMDGPU_INFO_VRAM_GTT;
+
+			ret = drmCommandWrite(drm_fd, DRM_AMDGPU_INFO,
+						&request, sizeof(request));
+			vramsize = vram_gtt.vram_size;
+#else
+			printf(_("amdgpu DRM driver is used, but amdgpu VRAM size reporting is not enabled\n"));
+#endif
+		}
 		if (ret) {
 			printf(_("Failed to get VRAM size, error %d\n"),
 				ret);
 			goto out;
 		}
-		vramsize = gem.vram_size;
 
 		ret = getvram();
 		if (ret == 0) {
+			if (strcmp(drm_name, "amdgpu") == 0) {
+#ifndef ENABLE_AMDGPU
+				printf(_("amdgpu DRM driver is used, but amdgpu VRAM usage reporting is not enabled\n"));
+#endif
+			}
 			printf(_("Failed to get VRAM usage, kernel likely too old\n"));
 			goto out;
 		}
@@ -192,20 +174,32 @@ unsigned int init_pci(unsigned char bus) {
 
 	pci_system_cleanup();
 
-	return dev->device_id;
+	return device_id;
 }
 
 unsigned long long getvram() {
 
-	int ret;
+	int ret = -1;
 	unsigned long long val = 0;
 
-	struct drm_radeon_info info;
-	memset(&info, 0, sizeof(info));
-	info.value = (unsigned long) &val;
-	info.request = RADEON_INFO_VRAM_USAGE;
+	if (strcmp(drm_name, "radeon") == 0) {
+		struct drm_radeon_info info;
+		memset(&info, 0, sizeof(info));
+		info.value = (unsigned long) &val;
+		info.request = RADEON_INFO_VRAM_USAGE;
 
-	ret = drmCommandWriteRead(drm_fd, DRM_RADEON_INFO, &info, sizeof(info));
+		ret = drmCommandWriteRead(drm_fd, DRM_RADEON_INFO, &info, sizeof(info));
+	} else if (strcmp(drm_name, "amdgpu") == 0) {
+#ifdef ENABLE_AMDGPU
+		struct drm_amdgpu_info request;
+		memset(&request, 0, sizeof(request));
+		request.return_pointer = (unsigned long) &val;
+		request.return_size = sizeof(val);
+		request.query = AMDGPU_INFO_VRAM_USAGE;
+
+		ret = drmCommandWrite(drm_fd, DRM_AMDGPU_INFO, &request, sizeof(request));
+#endif
+	}
 	if (ret) return 0;
 
 	return val;
