@@ -57,5 +57,9 @@ const char * const family_str[] = {
 	str(HAWAII),
 	str(TOPAZ),
 	str(TONGA),
+	str(FIJI),
 	str(CARRIZO),
+	str(STONEY),
+	str(POLARIS11),
+	str(POLARIS10),
 };
