@@ -498,8 +498,32 @@ CHIPSET(0x692F, 0x692F, TONGA)
 CHIPSET(0x6930, 0x6930, TONGA)
 CHIPSET(0x6938, 0x6938, TONGA)
 CHIPSET(0x6939, 0x6939, TONGA)
+CHIPSET(0x7300, 0x7300, FIJI)
 CHIPSET(0x9870, 0x9870, CARRIZO)
 CHIPSET(0x9874, 0x9874, CARRIZO)
 CHIPSET(0x9875, 0x9875, CARRIZO)
 CHIPSET(0x9876, 0x9876, CARRIZO)
 CHIPSET(0x9877, 0x9877, CARRIZO)
+CHIPSET(0x98E4, 0x98E4, STONEY)
+
+CHIPSET(0x67E0, 0x67E0, POLARIS11)
+CHIPSET(0x67E3, 0x67E3, POLARIS11)
+CHIPSET(0x67E8, 0x67E8, POLARIS11)
+CHIPSET(0x67EB, 0x67EB, POLARIS11)
+CHIPSET(0x67EF, 0x67EF, POLARIS11)
+CHIPSET(0x67FF, 0x67FF, POLARIS11)
+CHIPSET(0x67E1, 0x67E1, POLARIS11)
+CHIPSET(0x67E7, 0x67E7, POLARIS11)
+CHIPSET(0x67E9, 0x67E9, POLARIS11)
+
+CHIPSET(0x67C0, 0x67C0, POLARIS10)
+CHIPSET(0x67C1, 0x67C1, POLARIS10)
+CHIPSET(0x67C2, 0x67C2, POLARIS10)
+CHIPSET(0x67C4, 0x67C4, POLARIS10)
+CHIPSET(0x67C7, 0x67C7, POLARIS10)
+CHIPSET(0x67DF, 0x67DF, POLARIS10)
+CHIPSET(0x67C8, 0x67C8, POLARIS10)
+CHIPSET(0x67C9, 0x67C9, POLARIS10)
+CHIPSET(0x67CA, 0x67CA, POLARIS10)
+CHIPSET(0x67CC, 0x67CC, POLARIS10)
+CHIPSET(0x67CF, 0x67CF, POLARIS10)
